import { dirname, resolve } from 'node:path';
import { unixify } from '../util.mjs';
export const fixModuleReferences = (ctx) => {
    const { contents, filename, filenames, options: { cwd }, ignore } = ctx;
    const _contents = contents.replace(/((?:\s|^)import\s+|\s+from\s+|\W(?:import|require)\s*\()(["'])([^"']+\/[^"']+|\.{1,2})\/?(["'])/g, (_matched, control, q1, from, q2) => `${control}${q1}${ignore.includes(from) ? from : resolveDependency(filename, from, filenames, cwd)}${q2}`);
    return Object.assign(Object.assign({}, ctx), { contents: _contents });
};
export const resolveDependency = (parent, nested, files, cwd) => {
    const dir = dirname(parent);
    const nmdir = resolve(cwd, 'node_modules');
    const bases = /^\..+\.[^./\\]+$/.test(nested)
        ? [nested, nested.replace(/\.[^./\\]+$/, '')]
        : [nested];
    const variants = ['.js', '.cjs', '.mjs'].reduce((m, e) => {
        bases.forEach((v) => m.push(`${v}${e}`, `${v}/index${e}`));
        return m;
    }, []);
    return (variants.find((f) => files.includes(unixify(resolve(dir, f)))) ||
        variants.find((f) => files.includes(unixify(resolve(nmdir, f)))) ||
        nested);
};
